(function(){
	//Public API
	window.AF = {
		log:true,
		init: init,
		displaySpread: displaySpread
		
	};
	
	//Notifications 
	var ON_LOAD = 'onload'; // window onload event
	var READY = 'ready';    // paneling loaded
	var SPREAD_DISPLAYED = 'spreadDisplayed';
	
	var spreads = {};
	var spreadDisplayer;
	var currentSpreadIndex;
	
	//Load paneling and translation json once
	function init(panelingUrl, translationUrl, fontFolderUrl, fontScale){
		fontScale = fontScale || 1;
		
		if(AF.log) console.log('[main] init -> ' + 'panelingUrl: ' + panelingUrl + ', translationUrl ' + translationUrl + ', fontFolderUrl: ' + fontFolderUrl + ', fontScale: ' + fontScale);
		//Ideally, if we could have the full font list, we could preload them here

        var errorFct = function(jqXHR, textStatus, errorThrown){
                            console.log('	> error status: ' + textStatus + ', error: ' + errorThrown);
                          }

		var successPanelingFct = function(panelingData){
            if(AF.log) console.log('	> paneling loaded' + panelingData);

            var successTranslationFct = function(translationData){
                if(AF.log) console.log('	> translation loaded' + translationData);
                //build player model from raw data => match bubbles and their translations
                //and get used fonts
                var fontFamilies = {};
                spreads = panelingData.spreads;
                _.each(spreads, function(spread, spreadIndex){
                    //TODO ?
                    spread.pages = [];
                    spread.index = spreadIndex;

                    _.each(spread.panels, function(panel){
                        var bubbleList = [];
                        _.each(panel.bubbles, function(bubble, bubbleId){
                            //group bubble with its translation
                            var spreadTranslation = translationData.spreads[spreadIndex];
                            if(spreadTranslation){
                                bubble.translation = spreadTranslation[bubble.id];
                                if(bubble.translation && bubble.translation.textStyle){
                                    fontFamilies[bubble.translation.textStyle.fontFamily] = true;
                                    bubble.translation.textStyle.fontSize *= fontScale;
                                }
                            }
                            //transform bubbles: map => array
                            bubble.id = bubbleId;
                            bubbleList.push(bubble);
                        });
                        panel.bubbles = bubbleList;
                    });
                });

                //load fonts
                var fontManager = angular.getDependency('fontManager');
                var fontList = _.keys(fontFamilies);
                fontManager.loadFonts(fontList, fontFolderUrl, function(error){
                    if(AF.log) console.log('	> fonts loaded : ' + fontList);
                    if(error){
                        //TODO: warn reader ?
                        console.log('	> ERROR');
                    }
                    notify(READY);
                });
            };

//            jQuery.getJSON(translationUrl, successTranslationFct);
            $.ajax({
                  type: 'GET',
                  url: translationUrl,
//                  dataType: "jsonp",
                  dataType: "json",
                  data: {
                    format: "json"
                  },
//                  jsonp: "callback",
//                  crossDomain: true,
                  cache: false,
                  success: successTranslationFct,
                  error: errorFct
                });
        };


        $.ajax({
              type: 'GET',
              url: panelingUrl,
//              dataType: "jsonp",
              dataType: "json",
              data: {
                format: "json"
              },
//              jsonp: "callback",
//              crossDomain: true,
              cache: false,
              success: successPanelingFct,
              error: errorFct
            });

//		jQuery.getJSON(panelingUrl, successPanelingFct);
	};
	
	//displays the given spread and notifies when screenshot can be taken
	function displaySpread(spreadIndex, width, height, pageUrls){
		if(AF.log) console.log('[main] displaySpread : ' + spreadIndex);
		
		if(!spreadDisplayer){
			spreadDisplayer = angular.getDependency('spreadDisplayer');
		}
		
		currentSpreadIndex = spreadIndex;
//		if(AF.log) console.log('  > spreads size : ' + spreads.length);
		var spread = spreads[spreadIndex];
		
		//Note: this should be in spread.json but it is not there at the moment 
		spread.width = width;
		spread.height = height;
		if(pageUrls){
			spread.pages = _.map(pageUrls, function(pageUrl, index){
				var page = {
					x: 0,
					y: 0,
					width: 1,
					height: 1,
					image: {
						path: pageUrl
					}
				};
				if(pageUrls.length > 1){
					page.width = 0.5;
					if(index > 0){
						page.x = 0.5;
					}
				}
				
				return page;
			});
		}
		
		spreadDisplayer.display(spread, 
			function(){
				if(currentSpreadIndex == spreadIndex){
//					if(AF.log) console.log('	> spread displayed : ' + spreadIndex);

					notifyDisplayed();
				}
			});
	};
	
	function notify(){
		var args = Array.prototype.slice.call(arguments);
		if(AF.log) console.log('[main] notify' + args);
		
		//for desktop / debug
		if(AF.handleNotification){
			AF.handleNotification(args);
		}
		
		//for Android
		if(window.Android && typeof Android.handleNotification == "function"){
			//Android need all the args in a single JSON (and we avoid serializing args twice)
			var jsonArgs = JSON.stringify(args);
			Android.handleNotification(jsonArgs);
		}
	}
	
	function notifyDisplayed(){
		//force reflow
		document.body.style.display='none';
		document.body.offsetHeight; // no need to store this anywhere, the reference is enough
		document.body.style.display='';
		
		//notify reader (make sure to wait after frame redraw)
		window.requestAnimationFrame(function(){


//		var canvas = document.getElementById("box");
//		var dataUrl = canvas.toDataURL();

            notify(SPREAD_DISPLAYED, currentSpreadIndex);

//		notify(SPREAD_DISPLAYED, currentSpreadIndex, dataUrl.replace("data:image/png;base64,", ""));
		});
	}

	window.onload = function(){
		notify(ON_LOAD);
	};
})();